# Generated by Django 2.2.16 on 2020-09-16 18:39

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="General",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "permissions": (("basic_access", "Can access this app"),),
                "managed": False,
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="Distribution",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        help_text="Name of this package", max_length=255, unique=True
                    ),
                ),
                (
                    "description",
                    models.TextField(
                        default="", help_text="Description of this package"
                    ),
                ),
                (
                    "apps",
                    models.TextField(
                        default="",
                        help_text="List of installed Django apps included in this package (JSON)",
                    ),
                ),
                (
                    "has_installed_apps",
                    models.BooleanField(
                        db_index=True,
                        default=False,
                        help_text="Whether this package has installed Django apps",
                    ),
                ),
                (
                    "used_by",
                    models.TextField(
                        default="",
                        help_text="List of other distribution packages using this package (JSON)",
                    ),
                ),
                (
                    "installed_version",
                    models.CharField(
                        default="",
                        help_text="Currently installed version of this package",
                        max_length=64,
                    ),
                ),
                (
                    "latest_version",
                    models.CharField(
                        default="",
                        help_text="Latest stable version available for this package",
                        max_length=64,
                    ),
                ),
                (
                    "is_outdated",
                    models.BooleanField(
                        db_index=True,
                        default=None,
                        help_text="A package is outdated when there is a newer stable version available",
                        null=True,
                    ),
                ),
                (
                    "website_url",
                    models.TextField(
                        default="", help_text="URL to the home page of this package"
                    ),
                ),
                (
                    "updated_at",
                    models.DateTimeField(
                        auto_now=True,
                        help_text="Date & time this data was last updated",
                    ),
                ),
            ],
        ),
    ]
